# -*- coding: utf-8 -*-
#
# File: test_searchitems.py
#
# Copyright (c) 2013 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from DateTime import DateTime
from imio.helpers.cache import cleanRamCacheFor
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import deserialize
from imio.pm.ws.tests.WS4PMTestCase import serializeRequest
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import searchItemsRequest
from imio.pm.ws.WS4PM_client import searchItemsResponse
from time import localtime
from ZSI.TCtimes import gDateTime

import ZSI


class testSOAPSearchItems(WS4PMTestCase):
    """
        Tests the soap.searchItemsRequest method by accessing the real SOAP service
    """

    def test_ws_searchItemsRequest(self):
        """
          Test that searching with given parameters returns valuable informations
        """
        # by default no item exists
        self.changeUser('pmCreator1')
        self.assertEqual(len(self.portal.portal_catalog(portal_type='MeetingItemPga')), 0)
        # prepare data for a default item
        req = self._prepareCreationData()
        req._creationData._externalIdentifier = 'my_external_app_identifier'
        # use the SOAP service to create one
        newItem, response = self._createItem(req)
        # make sure created enough in the past or sort_on 'created' returns random result
        newItem.setCreationDate(DateTime() - 5)
        newItem.reindexObject()
        newItemUID = newItem.UID()
        # externalIdentifier is actually set
        self.assertEqual(newItem.externalIdentifier, 'my_external_app_identifier')
        # now an item exists, get informations about it
        req = searchItemsRequest()
        req._Title = 'item'
        req._getCategory = 'development'
        # Serialize the request so it can be easily tested
        request = serializeRequest(req)
        # This is what the sent enveloppe should looks like
        expected = """<SOAP-ENV:Envelope xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">""" \
            """<SOAP-ENV:Header></SOAP-ENV:Header>""" \
            """<SOAP-ENV:Body xmlns:ns1="http://ws4pm.imio.be"><ns1:searchItemsRequest>""" \
            """<Title>%s</Title><getCategory>%s</getCategory></ns1:searchItemsRequest>""" \
            """</SOAP-ENV:Body></SOAP-ENV:Envelope>""" % (req._Title, req._getCategory)
        result = """%s""" % request
        self.assertEqual(expected, result)
        # now really use the SOAP method to get informations about the item
        resp = self._searchItems(req)
        # the item is not in a meeting so the meeting date is 1950-01-01
        expected = """<ns1:searchItemsResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{0}</UID>
    <id>my-new-item-title</id>
    <title>My new item title</title>
    <creator>pmCreator1</creator>
    <creation_date>{1}</creation_date>
    <modification_date>{2}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>itemcreated</review_state>
    <meeting_date>1950-01-01T00:00:00.006Z</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmCreator1/mymeetings/plonegov-assembly/my-new-item-title</absolute_url>
    <externalIdentifier>my_external_app_identifier</externalIdentifier>
    <extraInfos/>
  </itemInfo>
</ns1:searchItemsResponse>
""".format(newItemUID,
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.modified())))
        self.assertEqual(expected, resp)
        # if the item is in a meeting, the result is a bit different because
        # we have valid informations about the meeting_date
        # use the 'plonegov-assembly' MeetingConfig that use real categories,
        # not useGroupsAsCategories
        self.changeUser('pmManager')
        meeting = self._createMeetingWithItems()
        itemInMeeting = meeting.get_items(ordered=True)[0]
        # by default, PloneMeeting creates item without title/description/decision...
        itemInMeeting.setTitle('My new item title')
        itemInMeeting.setDescription('<p>Description</p>', mimetype='text/x-html-safe')
        itemInMeeting.setDecision('<p>Décision</p>', mimetype='text/x-html-safe')
        itemInMeeting.reindexObject()
        req._Title = 'item title'
        req._getCategory = ''
        resp = self._searchItems(req)
        itemInMeetingUID = itemInMeeting.UID()
        meetingDate = gDateTime.get_formatted_content(gDateTime(), meeting.date.utctimetuple())
        # searching for items can returns several items
        # for example here, searching for 'item title' in existing items title will returns 2 items...
        expected = """<ns1:searchItemsResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{0}</UID>
    <id>my-new-item-title</id>
    <title>My new item title</title>
    <creator>pmCreator1</creator>
    <creation_date>{1}</creation_date>
    <modification_date>{2}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription>&lt;p&gt;Detailed description&lt;/p&gt;</detailedDescription>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>itemcreated</review_state>
    <meeting_date>1950-01-01T00:00:00.006Z</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmCreator1/mymeetings/plonegov-assembly/my-new-item-title</absolute_url>
    <externalIdentifier>my_external_app_identifier</externalIdentifier>
    <extraInfos/>
  </itemInfo>
  <itemInfo xsi:type="ns1:ItemInfo">
    <UID>{3}</UID>
    <id>item-2</id>
    <title>My new item title</title>
    <creator>pmManager</creator>
    <creation_date>{4}</creation_date>
    <modification_date>{5}</modification_date>
    <category>development</category>
    <description>&lt;p&gt;Description&lt;/p&gt;</description>
    <detailedDescription/>
    <decision>&lt;p&gt;Décision&lt;/p&gt;</decision>
    <preferredMeeting/>
    <preferred_meeting_date>1950-01-01T00:00:00.006Z</preferred_meeting_date>
    <review_state>presented</review_state>
    <meeting>{6}</meeting>
    <meeting_date>{7}</meeting_date>
    <absolute_url>http://nohost/plone/Members/pmManager/mymeetings/plonegov-assembly/item-2</absolute_url>
    <externalIdentifier/>
    <extraInfos/>
  </itemInfo>
</ns1:searchItemsResponse>
""".format(newItemUID,
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(newItem.modified())),
                itemInMeetingUID,
                gDateTime.get_formatted_content(gDateTime(), localtime(itemInMeeting.created())),
                gDateTime.get_formatted_content(gDateTime(), localtime(itemInMeeting.modified())),
                meeting.UID(),
                meetingDate)
        self.assertEqual(expected, resp)
        # if the search params do not return an existing UID, the response is empty
        req._Title = 'aWrongTitle'
        resp = self._searchItems(req)
        expected = """<ns1:searchItemsResponse xmlns:ns1="http://ws4pm.imio.be" """ \
            """xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" """ \
            """xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" """ \
            """xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" """ \
            """xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"/>
"""
        self.assertEqual(resp, expected)
        # if not search params is pass, a ZSI.Fault is raised
        req._Title = ''
        responseHolder = searchItemsResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, 'Define at least one search parameter!')
        # if a 'meetingConfigId' is passed, items of this meetingConfig are taken into account
        # create an item for 'plonemeeting-assembly' with same data as one created for 'plonegov-assembly' here above
        req = self._prepareCreationData()
        req._meetingConfigId = 'plonemeeting-assembly'
        # in 'plonemeeting-assembly', the category is not used, useGroupsAsCategories is True
        req._creationData._category = ''
        newItem, response = self._createItem(req)
        pmItem = self.portal.portal_catalog(UID=response._UID)[0].getObject()
        pmItemUID = pmItem.UID()
        # searching items with Title like 'item title' returns the 3 created elements
        req = searchItemsRequest()
        req._Title = 'item title'
        responseHolder = searchItemsResponse()
        response = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        resp = deserialize(response)
        self.failUnless(itemInMeetingUID in resp and newItemUID in resp and pmItemUID in resp)
        req._meetingConfigId = 'plonemeeting-assembly'
        response = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        resp = deserialize(response)
        self.failUnless(itemInMeetingUID not in resp and newItemUID not in resp and pmItemUID in resp)
        # passing a wrong meetingConfigId will raise a ZSI.Fault
        req._meetingConfigId = 'wrongMeetingConfigId'
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        self.assertEqual(cm.exception.string, "Unknown meetingConfigId : 'wrongMeetingConfigId'!")

    def test_ws_searchItemsInTheNameOf(self):
        """
          Test that searching items inTheNameOf antother user works
          Create 2 items, one by 'pmCreator1', member of the 'developers' group
          and one by 'pmCreator2', member of the 'vendors' group
          Item 1 will be viewable :
          - by 'pmManager' and 'pmCreator1'
          Item 1 will NOT be viewable :
          - while getting informations in the name of 'pmCreator2'
            that is not in the 'developers' group
          Item 2 will be viewable :
          - by 'pmManager' and 'pmCreator2'
          Item 2 will NOT be viewable :
          - while getting informations in the name of 'pmCreator1'
            that is not in the 'vendors' group
        """
        # put pmManager in the 'vendors_creators' so he can have
        # access to itemcreated items of 'pmCreator2'
        self.portal.portal_groups.addPrincipalToGroup('pmManager', self.vendors_creators)
        SAME_TITLE = 'sameTitleForBothItems'
        # create an item by 'pmCreator1'
        self.changeUser('pmCreator1')
        item1 = self.create('MeetingItem')
        item1.setTitle(SAME_TITLE)
        item1.reindexObject(idxs=['Title', ])
        # create an item by 'pmCreator2'
        self.changeUser('pmCreator2')
        item2 = self.create('MeetingItem')
        item2.setTitle(SAME_TITLE)
        item2.reindexObject(idxs=['Title', ])
        req = searchItemsRequest()
        req._inTheNameOf = None
        req._Title = SAME_TITLE
        responseHolder = searchItemsResponse()
        # 'pmCreator1' can get infos about item1
        self.changeUser('pmCreator1')
        result = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        # only one result and about item1
        self.assertEqual(result._itemInfo[0].UID, item1.UID())
        self.assertEqual(len(result._itemInfo), 1)
        # 'pmCreator2' can get infos about item2
        self.changeUser('pmCreator2')
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        result = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        # only one result and about item2
        self.assertEqual(result._itemInfo[0].UID, item2.UID())
        self.assertEqual(len(result._itemInfo), 1)
        # None of 'pmCreatorx' can searchItems inTheNameOf
        req._inTheNameOf = 'pmCreator1'
        self.changeUser('pmCreator1')
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            "You need to be 'Manager' or 'MeetingManager' to get item informations 'inTheNameOf'!")
        req._inTheNameOf = 'pmCreator2'
        self.changeUser('pmCreator2')
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        self.assertEqual(
            cm.exception.string,
            "You need to be 'Manager' or 'MeetingManager' to get item informations 'inTheNameOf'!")
        # now working examples with a 'Manager'
        self.changeUser('pmManager')
        req._inTheNameOf = None
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        result = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        # both items are returned
        self.assertEqual(len(result._itemInfo), 2)
        # returned items are item1 and item2
        createdItemsUids = set((item1.UID(), item2.UID()))
        resultUids = set((result._itemInfo[0].UID, result._itemInfo[1].UID))
        self.assertEqual(createdItemsUids, resultUids)
        # as we switch user while using inTheNameOf, make sure we have
        # falled back to original user
        self.assertEqual(self.portal.portal_membership.getAuthenticatedMember().getId(), 'pmManager')
        # now searchItems inTheNameOf 'pmCreator1'
        req._inTheNameOf = 'pmCreator1'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        result = SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)
        self.assertEqual(len(result._itemInfo), 1)
        self.assertEqual(result._itemInfo[0].UID, item1.UID())
        # now searchItems inTheNameOf 'pmCreator2'
        req._inTheNameOf = 'pmCreator2'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        result = SOAPView(self.portal, req).getItemInfosRequest(req, responseHolder)
        self.assertEqual(len(result._itemInfo), 1)
        self.assertEqual(result._itemInfo[0].UID, item2.UID())
        self._check_after_inTheNameOf()

    def test_ws_searchItemsMeetingUID(self):
        """Test search param meeting_uid."""
        self.changeUser('pmManager')
        meeting = self._createMeetingWithItems()
        req = searchItemsRequest()
        responseHolder = searchItemsResponse()
        # unknown meeting UID
        self.changeUser('pmCreator1')
        req._meeting_uid = 'unknown_uid'
        result = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        self.assertEqual(len(result._itemInfo), 0)
        req._meeting_uid = meeting.UID()
        result = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        # only get items user may view
        self.assertEqual(len(result._itemInfo), 2)
        self.changeUser('pmManager')
        result = SOAPView(self.portal, req).searchItemsRequest(req, responseHolder)
        self.assertEqual(len(result._itemInfo), 5)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPSearchItems, prefix='test_ws_'))
    return suite



# -*- coding: utf-8 -*-
"""
"""
from ZSI.schema import GTD
from imio.pm.ws.WS4PM_types import *


AnnexInfo = GTD('http://ws4pm.imio.be', 'AnnexInfo')('').pyclass
BasicInfo = GTD('http://ws4pm.imio.be', 'BasicInfo')('').pyclass
ConfigInfo = GTD('http://ws4pm.imio.be', 'ConfigInfo')('').pyclass
GroupInfo = GTD('http://ws4pm.imio.be', 'GroupInfo')('').pyclass
ItemInfo = GTD('http://ws4pm.imio.be', 'ItemInfo')('').pyclass
MeetingInfo = GTD('http://ws4pm.imio.be', 'MeetingInfo')('').pyclass
TemplateInfo = GTD('http://ws4pm.imio.be', 'TemplateInfo')('').pyclass



# -*- coding: utf-8 -*-
#
# File: test_connection.py
#
# Copyright (c) 2013 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from AccessControl import Unauthorized
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import testConnectionRequest
from imio.pm.ws.WS4PM_client import testConnectionResponse
from plone import api
from plone.app.testing import logout


class testSOAPConnection(WS4PMTestCase):
    """
        Tests the soap.connectionRequest method by accessing the real SOAP service
    """

    def test_ws_connectionRequest(self):
        """
          Test that we can connect to the webservice if we are authenticated in PloneMeeting
        """
        # try without being connected
        logout()
        req = testConnectionRequest()
        responseHolder = testConnectionResponse()
        self.assertRaises(Unauthorized, SOAPView(self.portal, req).testConnectionRequest, req, responseHolder)
        # now try with a connected user
        self.changeUser('pmManager')
        response = SOAPView(self.portal, req).testConnectionRequest(req, responseHolder)
        self.assertEquals(response._connectionState, True)

    def test_ws_connection_version(self):
        """
          The testConnection method will return the WS version
        """
        # try without being connected
        self.changeUser('pmManager')
        req = testConnectionRequest()
        responseHolder = testConnectionResponse()
        response = SOAPView(self.portal, req).testConnectionRequest(req, responseHolder)

        version = api.env.get_distribution('imio.pm.ws')._version
        self.assertEquals(response._version, version)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPConnection, prefix='test_ws_'))
    return suite



# -*- coding: utf-8 -*-
"""
"""
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserRequest


class IWS4PMLayer(IBrowserRequest):
    """
      Define a layer so the WDSL and schemaextender are only available when the BrowserLayer is installed
    """


class ITestConnectionRequest(Interface):
    """
    Marker interface for change request
    """


class ICheckIsLinkedRequest(Interface):
    """
    Marker interface for change request
    """


class IConfigInfosRequest(Interface):
    """
    Marker interface for change request
    """


class IUserInfosRequest(Interface):
    """
    Marker interface for change request
    """


class IItemInfosRequest(Interface):
    """
    Marker interface for change request
    """


class ISingleItemInfosRequest(Interface):
    """
    Marker interface for change request
    """


class IItemTemplateRequest(Interface):
    """
    Marker interface for change request
    """


class ISearchItemsRequest(Interface):
    """
    Marker interface for change request
    """


class IMeetingsAcceptingItemsRequest(Interface):
    """
    Marker interface for change request
    """


class ICreateItemRequest(Interface):
    """
    Marker interface for change request
    """


class IExternalIdentifierable(Interface):
    """
    Marker interface for externalIdentifier field schema extender
    """



##################################################
# file: WS4PM_types.py
#
# schema types generated by "ZSI.generate.wsdl2python.WriteServiceModule"
#    /srv/ZSI/bin/wsdl2py dumpedWSDL.txt -o . -b
#
##################################################

import ZSI
import ZSI.TCcompound
from ZSI.schema import LocalElementDeclaration, ElementDeclaration, TypeDefinition, GTD, GED
from ZSI.generate.pyclass import pyclass_type

##############################
# targetNamespace
# http://ws4pm.imio.be
##############################

class ns0:
    targetNamespace = "http://ws4pm.imio.be"

    class TestConnectionRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "TestConnectionRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.TestConnectionRequest_Def.schema
            TClist = [ZSI.TC.String(pname="dummy", aname="_dummy", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._dummy = "dummy"
                    return
            Holder.__name__ = "TestConnectionRequest_Holder"
            self.pyclass = Holder

    class CheckIsLinkedRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "CheckIsLinkedRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.CheckIsLinkedRequest_Def.schema
            TClist = [ZSI.TC.String(pname="meetingConfigId", aname="_meetingConfigId", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="externalIdentifier", aname="_externalIdentifier", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._meetingConfigId = None
                    self._externalIdentifier = None
                    return
            Holder.__name__ = "CheckIsLinkedRequest_Holder"
            self.pyclass = Holder

    class ConfigInfosRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "ConfigInfosRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ConfigInfosRequest_Def.schema
            TClist = [ZSI.TC.Boolean(pname="showCategories", aname="_showCategories", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="userToShowCategoriesFor", aname="_userToShowCategoriesFor", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._showCategories = 0
                    self._userToShowCategoriesFor = None
                    return
            Holder.__name__ = "ConfigInfosRequest_Holder"
            self.pyclass = Holder

    class UserInfosRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "UserInfosRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.UserInfosRequest_Def.schema
            TClist = [ZSI.TC.String(pname="userId", aname="_userId", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showGroups", aname="_showGroups", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="suffix", aname="_suffix", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._userId = None
                    self._showGroups = 0
                    self._suffix = None
                    return
            Holder.__name__ = "UserInfosRequest_Holder"
            self.pyclass = Holder

    class ItemInfosRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "ItemInfosRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ItemInfosRequest_Def.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showExtraInfos", aname="_showExtraInfos", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showAnnexes", aname="_showAnnexes", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="include_annex_binary", aname="_include_annex_binary", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","List",lazy=False)(pname="allowed_annexes_types", aname="_allowed_annexes_types", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showAssembly", aname="_showAssembly", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showTemplates", aname="_showTemplates", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showEmptyValues", aname="_showEmptyValues", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="inTheNameOf", aname="_inTheNameOf", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._showExtraInfos = 0
                    self._showAnnexes = 0
                    self._include_annex_binary = 1
                    self._allowed_annexes_types = []
                    self._showAssembly = 0
                    self._showTemplates = 0
                    self._showEmptyValues = 1
                    self._inTheNameOf = None
                    return
            Holder.__name__ = "ItemInfosRequest_Holder"
            self.pyclass = Holder

    class SingleItemInfosRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "SingleItemInfosRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.SingleItemInfosRequest_Def.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showExtraInfos", aname="_showExtraInfos", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showAnnexes", aname="_showAnnexes", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="include_annex_binary", aname="_include_annex_binary", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","List",lazy=False)(pname="allowed_annexes_types", aname="_allowed_annexes_types", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showAssembly", aname="_showAssembly", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showTemplates", aname="_showTemplates", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="showEmptyValues", aname="_showEmptyValues", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="inTheNameOf", aname="_inTheNameOf", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._showExtraInfos = 0
                    self._showAnnexes = 0
                    self._include_annex_binary = 1
                    self._allowed_annexes_types = []
                    self._showAssembly = 0
                    self._showTemplates = 0
                    self._showEmptyValues = 1
                    self._inTheNameOf = None
                    return
            Holder.__name__ = "SingleItemInfosRequest_Holder"
            self.pyclass = Holder

    class ItemTemplateRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "ItemTemplateRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ItemTemplateRequest_Def.schema
            TClist = [ZSI.TC.String(pname="itemUID", aname="_itemUID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="templateId", aname="_templateId", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="inTheNameOf", aname="_inTheNameOf", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._itemUID = None
                    self._templateId = None
                    self._inTheNameOf = None
                    return
            Holder.__name__ = "ItemTemplateRequest_Holder"
            self.pyclass = Holder

    class SearchItemsRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "SearchItemsRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.SearchItemsRequest_Def.schema
            TClist = [ZSI.TC.String(pname="Creator", aname="_Creator", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="Description", aname="_Description", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="SearchableText", aname="_SearchableText", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="Title", aname="_Title", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="UID", aname="_UID", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="getCategory", aname="_getCategory", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="getDecision", aname="_getDecision", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="getProposingGroup", aname="_getProposingGroup", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="portal_type", aname="_portal_type", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="review_state", aname="_review_state", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="meeting_uid", aname="_meeting_uid", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="meetingConfigId", aname="_meetingConfigId", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="externalIdentifier", aname="_externalIdentifier", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="inTheNameOf", aname="_inTheNameOf", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._Creator = None
                    self._Description = None
                    self._SearchableText = None
                    self._Title = None
                    self._UID = None
                    self._getCategory = None
                    self._getDecision = None
                    self._getProposingGroup = None
                    self._portal_type = None
                    self._review_state = None
                    self._meeting_uid = None
                    self._meetingConfigId = None
                    self._externalIdentifier = None
                    self._inTheNameOf = None
                    return
            Holder.__name__ = "SearchItemsRequest_Holder"
            self.pyclass = Holder

    class MeetingsAcceptingItemsRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "MeetingsAcceptingItemsRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.MeetingsAcceptingItemsRequest_Def.schema
            TClist = [ZSI.TC.String(pname="meetingConfigId", aname="_meetingConfigId", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="inTheNameOf", aname="_inTheNameOf", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._meetingConfigId = None
                    self._inTheNameOf = None
                    return
            Holder.__name__ = "MeetingsAcceptingItemsRequest_Holder"
            self.pyclass = Holder

    class CreateItemRequest_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "CreateItemRequest")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.CreateItemRequest_Def.schema
            TClist = [ZSI.TC.String(pname="meetingConfigId", aname="_meetingConfigId", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="proposingGroupId", aname="_proposingGroupId", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","CreationData",lazy=False)(pname="creationData", aname="_creationData", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="cleanHtml", aname="_cleanHtml", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","List",lazy=False)(pname="wfTransitions", aname="_wfTransitions", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="inTheNameOf", aname="_inTheNameOf", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._meetingConfigId = None
                    self._proposingGroupId = None
                    self._creationData = None
                    self._cleanHtml = 1
                    self._wfTransitions = []
                    self._inTheNameOf = None
                    return
            Holder.__name__ = "CreateItemRequest_Holder"
            self.pyclass = Holder

    class List_Def(ZSI.TC.String, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "List")
        def __init__(self, pname, **kw):
            ZSI.TC.String.__init__(self, pname, pyclass=None, **kw)
            class Holder(str):
                typecode = self
            self.pyclass = Holder

    class ConfigInfo_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "ConfigInfo")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ConfigInfo_Def.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="id", aname="_id", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="description", aname="_description", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","List",lazy=False)(pname="itemPositiveDecidedStates", aname="_itemPositiveDecidedStates", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","BasicInfo",lazy=False)(pname="categories", aname="_categories", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._id = None
                    self._title = None
                    self._description = None
                    self._itemPositiveDecidedStates = []
                    self._categories = []
                    return
            Holder.__name__ = "ConfigInfo_Holder"
            self.pyclass = Holder

    class GroupInfo_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "GroupInfo")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.GroupInfo_Def.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="id", aname="_id", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="description", aname="_description", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._id = None
                    self._title = None
                    self._description = None
                    return
            Holder.__name__ = "GroupInfo_Holder"
            self.pyclass = Holder

    class BasicInfo_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "BasicInfo")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.BasicInfo_Def.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="id", aname="_id", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="description", aname="_description", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._id = None
                    self._title = None
                    self._description = None
                    return
            Holder.__name__ = "BasicInfo_Holder"
            self.pyclass = Holder

    class CreationData_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "CreationData")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.CreationData_Def.schema
            TClist = [ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="category", aname="_category", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="description", aname="_description", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="detailedDescription", aname="_detailedDescription", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="motivation", aname="_motivation", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="decision", aname="_decision", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="preferredMeeting", aname="_preferredMeeting", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","List",lazy=False)(pname="associatedGroups", aname="_associatedGroups", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","List",lazy=False)(pname="groupsInCharge", aname="_groupsInCharge", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","List",lazy=False)(pname="optionalAdvisers", aname="_optionalAdvisers", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Boolean(pname="toDiscuss", aname="_toDiscuss", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="externalIdentifier", aname="_externalIdentifier", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","ExtraAttr",lazy=False)(pname="extraAttrs", aname="_extraAttrs", minOccurs=0, maxOccurs=10, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","AnnexInfo",lazy=False)(pname="annexes", aname="_annexes", minOccurs=0, maxOccurs=10, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._title = None
                    self._category = None
                    self._description = None
                    self._detailedDescription = None
                    self._motivation = None
                    self._decision = None
                    self._preferredMeeting = None
                    self._associatedGroups = []
                    self._groupsInCharge = []
                    self._optionalAdvisers = []
                    self._toDiscuss = None
                    self._externalIdentifier = None
                    self._extraAttrs = []
                    self._annexes = []
                    return
            Holder.__name__ = "CreationData_Holder"
            self.pyclass = Holder

    class ExtraAttr_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "ExtraAttr")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ExtraAttr_Def.schema
            TClist = [ZSI.TC.String(pname="key", aname="_key", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="value", aname="_value", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._key = None
                    self._value = None
                    return
            Holder.__name__ = "ExtraAttr_Holder"
            self.pyclass = Holder

    class AnnexInfo_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "AnnexInfo")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.AnnexInfo_Def.schema
            TClist = [ZSI.TC.String(pname="id", aname="_id", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="annexTypeId", aname="_annexTypeId", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="filename", aname="_filename", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.Base64String(pname="file", aname="_file", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._id = None
                    self._title = None
                    self._annexTypeId = None
                    self._filename = None
                    self._file = None
                    return
            Holder.__name__ = "AnnexInfo_Holder"
            self.pyclass = Holder

    class TemplateInfo_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "TemplateInfo")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.TemplateInfo_Def.schema
            TClist = [ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="templateFormat", aname="_templateFormat", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="templateId", aname="_templateId", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="templateFilename", aname="_templateFilename", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._title = None
                    self._templateFormat = None
                    self._templateId = None
                    self._templateFilename = None
                    return
            Holder.__name__ = "TemplateInfo_Holder"
            self.pyclass = Holder

    class ItemInfo_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "ItemInfo")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.ItemInfo_Def.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="id", aname="_id", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="creator", aname="_creator", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="creation_date", aname="_creation_date", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="modification_date", aname="_modification_date", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="category", aname="_category", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="description", aname="_description", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="detailedDescription", aname="_detailedDescription", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="decision", aname="_decision", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="preferredMeeting", aname="_preferredMeeting", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="preferred_meeting_date", aname="_preferred_meeting_date", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="review_state", aname="_review_state", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="meeting", aname="_meeting", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="meeting_date", aname="_meeting_date", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="item_assembly", aname="_item_assembly", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="absolute_url", aname="_absolute_url", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="externalIdentifier", aname="_externalIdentifier", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.AnyType(pname="extraInfos", aname="_extraInfos", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","AnnexInfo",lazy=False)(pname="annexes", aname="_annexes", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","TemplateInfo",lazy=False)(pname="templates", aname="_templates", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._id = None
                    self._title = None
                    self._creator = None
                    self._creation_date = None
                    self._modification_date = None
                    self._category = None
                    self._description = None
                    self._detailedDescription = None
                    self._decision = None
                    self._preferredMeeting = None
                    self._preferred_meeting_date = None
                    self._review_state = None
                    self._meeting = None
                    self._meeting_date = None
                    self._item_assembly = None
                    self._absolute_url = None
                    self._externalIdentifier = None
                    self._extraInfos = None
                    self._annexes = []
                    self._templates = []
                    return
            Holder.__name__ = "ItemInfo_Holder"
            self.pyclass = Holder

    class MeetingInfo_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
        schema = "http://ws4pm.imio.be"
        type = (schema, "MeetingInfo")
        def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
            ns = ns0.MeetingInfo_Def.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="date", aname="_date", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            self.attribute_typecode_dict = attributes or {}
            if extend: TClist += ofwhat
            if restrict: TClist = ofwhat
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._date = None
                    return
            Holder.__name__ = "MeetingInfo_Holder"
            self.pyclass = Holder

    class testConnectionRequest_Dec(ElementDeclaration):
        literal = "testConnectionRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'testConnectionRequest')
            kw["aname"] = "_testConnectionRequest"
            if ns0.TestConnectionRequest_Def not in ns0.testConnectionRequest_Dec.__bases__:
                bases = list(ns0.testConnectionRequest_Dec.__bases__)
                bases.insert(0, ns0.TestConnectionRequest_Def)
                ns0.testConnectionRequest_Dec.__bases__ = tuple(bases)

            ns0.TestConnectionRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "testConnectionRequest_Dec_Holder"

    class testConnectionResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "testConnectionResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.testConnectionResponse_Dec.schema
            TClist = [ZSI.TC.Boolean(pname="connectionState", aname="_connectionState", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="version", aname="_version", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'testConnectionResponse')
            kw["aname"] = "_testConnectionResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._connectionState = None
                    self._version = None
                    return
            Holder.__name__ = "testConnectionResponse_Holder"
            self.pyclass = Holder

    class checkIsLinkedRequest_Dec(ElementDeclaration):
        literal = "checkIsLinkedRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'checkIsLinkedRequest')
            kw["aname"] = "_checkIsLinkedRequest"
            if ns0.CheckIsLinkedRequest_Def not in ns0.checkIsLinkedRequest_Dec.__bases__:
                bases = list(ns0.checkIsLinkedRequest_Dec.__bases__)
                bases.insert(0, ns0.CheckIsLinkedRequest_Def)
                ns0.checkIsLinkedRequest_Dec.__bases__ = tuple(bases)

            ns0.CheckIsLinkedRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "checkIsLinkedRequest_Dec_Holder"

    class checkIsLinkedResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "checkIsLinkedResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.checkIsLinkedResponse_Dec.schema
            TClist = [ZSI.TC.Boolean(pname="isLinked", aname="_isLinked", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'checkIsLinkedResponse')
            kw["aname"] = "_checkIsLinkedResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._isLinked = None
                    return
            Holder.__name__ = "checkIsLinkedResponse_Holder"
            self.pyclass = Holder

    class getConfigInfosRequest_Dec(ElementDeclaration):
        literal = "getConfigInfosRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'getConfigInfosRequest')
            kw["aname"] = "_getConfigInfosRequest"
            if ns0.ConfigInfosRequest_Def not in ns0.getConfigInfosRequest_Dec.__bases__:
                bases = list(ns0.getConfigInfosRequest_Dec.__bases__)
                bases.insert(0, ns0.ConfigInfosRequest_Def)
                ns0.getConfigInfosRequest_Dec.__bases__ = tuple(bases)

            ns0.ConfigInfosRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "getConfigInfosRequest_Dec_Holder"

    class getConfigInfosResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "getConfigInfosResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.getConfigInfosResponse_Dec.schema
            TClist = [GTD("http://ws4pm.imio.be","ConfigInfo",lazy=False)(pname="configInfo", aname="_configInfo", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","GroupInfo",lazy=False)(pname="groupInfo", aname="_groupInfo", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'getConfigInfosResponse')
            kw["aname"] = "_getConfigInfosResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._configInfo = []
                    self._groupInfo = []
                    return
            Holder.__name__ = "getConfigInfosResponse_Holder"
            self.pyclass = Holder

    class getUserInfosRequest_Dec(ElementDeclaration):
        literal = "getUserInfosRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'getUserInfosRequest')
            kw["aname"] = "_getUserInfosRequest"
            if ns0.UserInfosRequest_Def not in ns0.getUserInfosRequest_Dec.__bases__:
                bases = list(ns0.getUserInfosRequest_Dec.__bases__)
                bases.insert(0, ns0.UserInfosRequest_Def)
                ns0.getUserInfosRequest_Dec.__bases__ = tuple(bases)

            ns0.UserInfosRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "getUserInfosRequest_Dec_Holder"

    class getUserInfosResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "getUserInfosResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.getUserInfosResponse_Dec.schema
            TClist = [ZSI.TC.String(pname="fullname", aname="_fullname", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="email", aname="_email", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","BasicInfo",lazy=False)(pname="groups", aname="_groups", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'getUserInfosResponse')
            kw["aname"] = "_getUserInfosResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._fullname = None
                    self._email = None
                    self._groups = []
                    return
            Holder.__name__ = "getUserInfosResponse_Holder"
            self.pyclass = Holder

    class getItemInfosRequest_Dec(ElementDeclaration):
        literal = "getItemInfosRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'getItemInfosRequest')
            kw["aname"] = "_getItemInfosRequest"
            if ns0.ItemInfosRequest_Def not in ns0.getItemInfosRequest_Dec.__bases__:
                bases = list(ns0.getItemInfosRequest_Dec.__bases__)
                bases.insert(0, ns0.ItemInfosRequest_Def)
                ns0.getItemInfosRequest_Dec.__bases__ = tuple(bases)

            ns0.ItemInfosRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "getItemInfosRequest_Dec_Holder"

    class getItemInfosResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "getItemInfosResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.getItemInfosResponse_Dec.schema
            TClist = [GTD("http://ws4pm.imio.be","ItemInfo",lazy=False)(pname="itemInfo", aname="_itemInfo", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'getItemInfosResponse')
            kw["aname"] = "_getItemInfosResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._itemInfo = []
                    return
            Holder.__name__ = "getItemInfosResponse_Holder"
            self.pyclass = Holder

    class getSingleItemInfosRequest_Dec(ElementDeclaration):
        literal = "getSingleItemInfosRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'getSingleItemInfosRequest')
            kw["aname"] = "_getSingleItemInfosRequest"
            if ns0.SingleItemInfosRequest_Def not in ns0.getSingleItemInfosRequest_Dec.__bases__:
                bases = list(ns0.getSingleItemInfosRequest_Dec.__bases__)
                bases.insert(0, ns0.SingleItemInfosRequest_Def)
                ns0.getSingleItemInfosRequest_Dec.__bases__ = tuple(bases)

            ns0.SingleItemInfosRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "getSingleItemInfosRequest_Dec_Holder"

    class getSingleItemInfosResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "getSingleItemInfosResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.getSingleItemInfosResponse_Dec.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="id", aname="_id", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="title", aname="_title", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="creator", aname="_creator", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="creation_date", aname="_creation_date", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="modification_date", aname="_modification_date", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="category", aname="_category", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="description", aname="_description", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="detailedDescription", aname="_detailedDescription", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="decision", aname="_decision", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="preferredMeeting", aname="_preferredMeeting", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="preferred_meeting_date", aname="_preferred_meeting_date", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="review_state", aname="_review_state", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="meeting", aname="_meeting", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TCtimes.gDateTime(pname="meeting_date", aname="_meeting_date", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="item_assembly", aname="_item_assembly", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="absolute_url", aname="_absolute_url", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="externalIdentifier", aname="_externalIdentifier", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.AnyType(pname="extraInfos", aname="_extraInfos", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","AnnexInfo",lazy=False)(pname="annexes", aname="_annexes", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")), GTD("http://ws4pm.imio.be","TemplateInfo",lazy=False)(pname="templates", aname="_templates", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'getSingleItemInfosResponse')
            kw["aname"] = "_getSingleItemInfosResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._id = None
                    self._title = None
                    self._creator = None
                    self._creation_date = None
                    self._modification_date = None
                    self._category = None
                    self._description = None
                    self._detailedDescription = None
                    self._decision = None
                    self._preferredMeeting = None
                    self._preferred_meeting_date = None
                    self._review_state = None
                    self._meeting = None
                    self._meeting_date = None
                    self._item_assembly = None
                    self._absolute_url = None
                    self._externalIdentifier = None
                    self._extraInfos = None
                    self._annexes = []
                    self._templates = []
                    return
            Holder.__name__ = "getSingleItemInfosResponse_Holder"
            self.pyclass = Holder

    class getItemTemplateRequest_Dec(ElementDeclaration):
        literal = "getItemTemplateRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'getItemTemplateRequest')
            kw["aname"] = "_getItemTemplateRequest"
            if ns0.ItemTemplateRequest_Def not in ns0.getItemTemplateRequest_Dec.__bases__:
                bases = list(ns0.getItemTemplateRequest_Dec.__bases__)
                bases.insert(0, ns0.ItemTemplateRequest_Def)
                ns0.getItemTemplateRequest_Dec.__bases__ = tuple(bases)

            ns0.ItemTemplateRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "getItemTemplateRequest_Dec_Holder"

    class getItemTemplateResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "getItemTemplateResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.getItemTemplateResponse_Dec.schema
            TClist = [ZSI.TC.Base64String(pname="file", aname="_file", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'getItemTemplateResponse')
            kw["aname"] = "_getItemTemplateResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._file = None
                    return
            Holder.__name__ = "getItemTemplateResponse_Holder"
            self.pyclass = Holder

    class searchItemsRequest_Dec(ElementDeclaration):
        literal = "searchItemsRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'searchItemsRequest')
            kw["aname"] = "_searchItemsRequest"
            if ns0.SearchItemsRequest_Def not in ns0.searchItemsRequest_Dec.__bases__:
                bases = list(ns0.searchItemsRequest_Dec.__bases__)
                bases.insert(0, ns0.SearchItemsRequest_Def)
                ns0.searchItemsRequest_Dec.__bases__ = tuple(bases)

            ns0.SearchItemsRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "searchItemsRequest_Dec_Holder"

    class searchItemsResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "searchItemsResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.searchItemsResponse_Dec.schema
            TClist = [GTD("http://ws4pm.imio.be","ItemInfo",lazy=False)(pname="itemInfo", aname="_itemInfo", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'searchItemsResponse')
            kw["aname"] = "_searchItemsResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._itemInfo = []
                    return
            Holder.__name__ = "searchItemsResponse_Holder"
            self.pyclass = Holder

    class meetingsAcceptingItemsRequest_Dec(ElementDeclaration):
        literal = "meetingsAcceptingItemsRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'meetingsAcceptingItemsRequest')
            kw["aname"] = "_meetingsAcceptingItemsRequest"
            if ns0.MeetingsAcceptingItemsRequest_Def not in ns0.meetingsAcceptingItemsRequest_Dec.__bases__:
                bases = list(ns0.meetingsAcceptingItemsRequest_Dec.__bases__)
                bases.insert(0, ns0.MeetingsAcceptingItemsRequest_Def)
                ns0.meetingsAcceptingItemsRequest_Dec.__bases__ = tuple(bases)

            ns0.MeetingsAcceptingItemsRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "meetingsAcceptingItemsRequest_Dec_Holder"

    class meetingsAcceptingItemsResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "meetingsAcceptingItemsResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.meetingsAcceptingItemsResponse_Dec.schema
            TClist = [GTD("http://ws4pm.imio.be","MeetingInfo",lazy=False)(pname="meetingInfo", aname="_meetingInfo", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'meetingsAcceptingItemsResponse')
            kw["aname"] = "_meetingsAcceptingItemsResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._meetingInfo = []
                    return
            Holder.__name__ = "meetingsAcceptingItemsResponse_Holder"
            self.pyclass = Holder

    class createItemRequest_Dec(ElementDeclaration):
        literal = "createItemRequest"
        schema = "http://ws4pm.imio.be"
        substitutionGroup = None
        def __init__(self, **kw):
            kw["pname"] = (u'http://ws4pm.imio.be', u'createItemRequest')
            kw["aname"] = "_createItemRequest"
            if ns0.CreateItemRequest_Def not in ns0.createItemRequest_Dec.__bases__:
                bases = list(ns0.createItemRequest_Dec.__bases__)
                bases.insert(0, ns0.CreateItemRequest_Def)
                ns0.createItemRequest_Dec.__bases__ = tuple(bases)

            ns0.CreateItemRequest_Def.__init__(self, **kw)
            if self.pyclass is not None: self.pyclass.__name__ = "createItemRequest_Dec_Holder"

    class createItemResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
        literal = "createItemResponse"
        schema = "http://ws4pm.imio.be"
        def __init__(self, **kw):
            ns = ns0.createItemResponse_Dec.schema
            TClist = [ZSI.TC.String(pname="UID", aname="_UID", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), ZSI.TC.String(pname="warnings", aname="_warnings", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded"))]
            kw["pname"] = (u'http://ws4pm.imio.be', u'createItemResponse')
            kw["aname"] = "_createItemResponse"
            self.attribute_typecode_dict = {}
            ZSI.TCcompound.ComplexType.__init__(self,None,TClist,inorder=0,**kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    self._UID = None
                    self._warnings = []
                    return
            Holder.__name__ = "createItemResponse_Holder"
            self.pyclass = Holder

# end class ns0 (tns: http://ws4pm.imio.be)



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from BeautifulSoup import BeautifulSoup
from collective.contact.plonegroup.utils import get_organizations
from collective.iconifiedcategory.utils import calculate_category_id
from DateTime import DateTime
from HTMLParser import HTMLParser
from imio.helpers.content import get_vocab
from imio.pm.ws.config import EXTERNAL_IDENTIFIER_FIELD_NAME
from imio.pm.ws.config import MAIN_DATA_FROM_ITEM_SCHEMA
from imio.pm.ws.config import POD_TEMPLATE_ID_PATTERN
from imio.pm.ws.soap.basetypes import AnnexInfo
from imio.pm.ws.soap.basetypes import BasicInfo
from imio.pm.ws.soap.basetypes import ConfigInfo
from imio.pm.ws.soap.basetypes import GroupInfo
from imio.pm.ws.soap.basetypes import ItemInfo
from imio.pm.ws.soap.basetypes import MeetingInfo
from imio.pm.ws.soap.basetypes import TemplateInfo
from lxml.html.clean import Cleaner
from magic import MagicException
from plone import api
from plone import namedfile
from plone.dexterity.utils import createContentInContainer
from Products.Archetypes.atapi import RichWidget
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from Products.PloneMeeting.browser.overrides import PMDocumentGeneratorLinksViewlet
from Products.PloneMeeting.config import ITEM_NO_PREFERRED_MEETING_VALUE
from Products.PloneMeeting.MeetingItem import MeetingItem
from Products.PloneMeeting.utils import add_wf_history_action
from Products.PloneMeeting.utils import get_annexes
from Products.PloneMeeting.utils import org_id_to_uid
from time import localtime
from zope.i18n import translate

import logging
import magic
import os
import ZSI


logger = logging.getLogger('WS4PM')

WRONG_HTML_WARNING = "HTML used for creating the item at '${item_path}' by '${creator}' was not valid. " \
                     "Used corrected HTML."
MIMETYPE_NOT_FOUND_OF_ANNEX_WARNING = "Mimetype could not be determined correctly for annex '${annex_path}' of " \
                                      "item '${item_path}', this annex was not added."
NO_EXTENSION_FOR_MIMETYPE_OF_ANNEX_WARNING = "No extension available in mimetypes_registry for mimetype '${mime}' " \
                                             "for annex '${annex_path}' of item '${item_path}', " \
                                             "this annex was not added."
MULTIPLE_EXTENSION_FOR_MIMETYPE_OF_ANNEX_WARNING = "Could not determine an extension to use for mimetype '${mime}', " \
                                                   "too many available, for annex '${annex_path}' of " \
                                                   "item '${item_path}', this annex was not added."
ITEM_SOAP_CREATED = "create_item_using_imio_pm_ws_soap"


def setup_user_in_the_name_of(portal, member):
    """ """
    # remove AUTHENTICATED_USER during adopt_user to avoid
    # breaking utils.get_current_user_id
    original_auth_user = portal.REQUEST.get("AUTHENTICATED_USER")
    if original_auth_user:
        portal.REQUEST["AUTHENTICATED_USER"] = None

    oldsm = getSecurityManager()
    newSecurityManager(portal.REQUEST, member)
    return oldsm, original_auth_user


def teardown_user_in_the_name_of(portal, oldsm, original_auth_user):
    """ """
    setSecurityManager(oldsm)
    portal.REQUEST["AUTHENTICATED_USER"] = original_auth_user


def raiseMsg(msg):
    """ """
    logger.error(msg)
    raise ZSI.Fault(ZSI.Fault.Client, msg)


class SOAPView(BrowserView):
    """
      class delivering SOAP methods for Products.PloneMeeting
    """

    def testConnectionRequest(self, request, response):
        '''
          This is the accessed SOAP method for testing the connection to the webservices
          This method is usefull for SOAP clients
        '''
        response._connectionState, response._version = self._testConnection()
        return response

    def checkIsLinkedRequest(self, request, response):
        '''
          This is the accessed SOAP method for checking if an element as already a given externalIdentifier
          This perform an unrestritedSearchResutls that is why it only returns True or False
          Only a 'Manager' or 'MeetingManager' can do this request
        '''
        response._isLinked = self._checkIsLinked(request._meetingConfigId, request._externalIdentifier)
        return response

    def getConfigInfosRequest(self, request, response):
        '''
          This is the accessed SOAP method for getting informations about the configuration
          This will return a list of key elements of the config with the type of element
        '''
        response._configInfo, response._groupInfo = \
            self._getConfigInfos(request._showCategories, request._userToShowCategoriesFor)
        return response

    def getUserInfosRequest(self, request, response):
        '''
          This is the accessed SOAP method for getting informations about an existing user
        '''
        response._fullname, response._email, response._groups = self._getUserInfos(request._userId,
                                                                                   request._showGroups,
                                                                                   request._suffix)
        return response

    def searchItemsRequest(self, request, response):
        '''
          This is the accessed SOAP method for searching items
        '''
        params = dict(request.__dict__)
        # remove the '_inTheNameOf' from searchParams as it is not a search parameter
        inTheNameOf = None
        if '_inTheNameOf' in params:
            inTheNameOf = params['_inTheNameOf']
            params.pop('_inTheNameOf')
        response._itemInfo = self._getItemInfos(params, inTheNameOf=inTheNameOf)
        return response

    def getItemInfosRequest(self, request, response):
        '''
          This is the accessed SOAP method for getting informations about an existing item
          This is an helper method when you just need to access an item you know the UID of
        '''
        params = dict(request.__dict__)
        # remove params that are not search parameter
        if '_showExtraInfos' in params:
            params.pop('_showExtraInfos')
        if '_showAnnexes' in params:
            params.pop('_showAnnexes')
        if '_allowed_annexes_types' in params:
            params.pop('_allowed_annexes_types')
        if '_include_annex_binary' in params:
            params.pop('_include_annex_binary')
        if '_showAssembly' in params:
            params.pop('_showAssembly')
        if '_showTemplates' in params:
            params.pop('_showTemplates')
        showEmptyValues = True
        if '_showEmptyValues' in params:
            showEmptyValues = params.pop('_showEmptyValues')
        inTheNameOf = None
        if '_inTheNameOf' in params:
            inTheNameOf = params.pop('_inTheNameOf')

        infos = self._getItemInfos(
            params,
            request.__dict__.get('_showExtraInfos', False),
            request.__dict__.get('_showAnnexes', False),
            request.__dict__.get('_allowed_annexes_types', []),
            request.__dict__.get('_include_annex_binary', True),
            request.__dict__.get('_showAssembly', False),
            request.__dict__.get('_showTemplates', False),
            inTheNameOf)
        if infos and not showEmptyValues:
            # remove empty data
            for k, v in infos[0].__dict__.items():
                if not v:
                    delattr(infos[0], k)
        response._itemInfo = infos
        return response

    def getSingleItemInfosRequest(self, request, response):
        '''
          This is the accessed SOAP method for getting informations about an existing item
          This is an helper method when you just need to access an item you know the UID of
        '''
        params = dict(request.__dict__)

        # remove params that are not search parameter
        if '_showExtraInfos' in params:
            params.pop('_showExtraInfos')
        if '_showAnnexes' in params:
            params.pop('_showAnnexes')
        if '_allowed_annexes_types' in params:
            params.pop('_allowed_annexes_types')
        if '_include_annex_binary' in params:
            params.pop('_include_annex_binary')
        if '_showAssembly' in params:
            params.pop('_showAssembly')
        if '_showTemplates' in params:
            params.pop('_showTemplates')
        showEmptyValues = True
        if '_showEmptyValues' in params:
            showEmptyValues = params.pop('_showEmptyValues')
        inTheNameOf = None
        if '_inTheNameOf' in params:
            inTheNameOf = params.pop('_inTheNameOf')

        infos = self._getItemInfos(
            params,
            request.__dict__.get('_showExtraInfos', False),
            request.__dict__.get('_showAnnexes', False),
            request.__dict__.get('_allowed_annexes_types', []),
            request.__dict__.get('_include_annex_binary', True),
            request.__dict__.get('_showAssembly', False),
            request.__dict__.get('_showTemplates', False),
            inTheNameOf)
        if infos:
            if not showEmptyValues:
                # remove empty data
                for k, v in infos[0].__dict__.items():
                    if not v:
                        delattr(infos[0], k)
            response.__dict__ = infos[0].__dict__
        else:
            response.__dict__ = {}
        return response

    def getItemTemplateRequest(self, request, response):
        '''
          This is the accessed SOAP method for getting a generated version of a given template
        '''
        response._file = self._getItemTemplate(request._itemUID,
                                               request._templateId,
                                               request._inTheNameOf)
        return response

    def meetingsAcceptingItemsRequest(self, request, response):
        '''
          This is the accessed SOAP method for getting meetings that accept items in a given meeting config
        '''
        response._meetingInfo = self._meetingsAcceptingItems(request._meetingConfigId,
                                                             request._inTheNameOf)
        return response

    def createItemRequest(self, request, response):
        '''
          This is the accessed SOAP method for creating an item
        '''
        response._UID, response._warnings = self._createItem(request._meetingConfigId,
                                                             request._proposingGroupId,
                                                             request._creationData,
                                                             request._cleanHtml,
                                                             request._wfTransitions,
                                                             request._inTheNameOf)
        return response

    def _testConnection(self):
        '''
          Test current connection state
        '''
        portal = self.context

        # in case this method is accessed without valid credrentials, raise Unauthorized
        if api.user.is_anonymous():
            raise Unauthorized

        logger.info('Test connection SOAP made at "%s".' % portal.absolute_url_path())
        version = api.env.get_distribution('imio.pm.ws')._version
        return True, version

    def _checkIsLinked(self, meetingConfigId, externalIdentifier):
        '''
          Test if an element in PloneMeeting is linked to given meetingConfig/externalIdentifier
        '''
        portal = self.context

        # this is only available to 'Manager' and 'MeetingManager'
        if not self._mayAccessAdvancedFunctionnalities():
            raiseMsg(
                "You need to be 'Manager' or 'MeetingManager' to check if an element is linked to an item!")

        # perform the unrestrictedSearchResult
        tool = api.portal.get_tool('portal_plonemeeting')
        # if a meetingConfigId is given, check that it exists
        query = {}
        if meetingConfigId:
            mc = getattr(tool, meetingConfigId, None)
            if not mc or not mc.meta_type == 'MeetingConfig':
                raiseMsg("Unknown meetingConfigId : '%s'!" % meetingConfigId)
            query['portal_type'] = mc.getItemTypeName()
        query['externalIdentifier'] = externalIdentifier
        catalog = api.portal.get_tool('portal_catalog')
        brains = catalog.unrestrictedSearchResults(**query)

        logger.info('checkIsLinked SOAP made at "%s".' % portal.absolute_url_path())
        if brains:
            return True
        return False

    def _getConfigInfos(self, showCategories=False, userToShowCategoriesFor=None):
        '''
          Returns key informations about the configuration : active Organizations and MeetingConfigs
          If p_showCategories is given, we return also available categories.  We return every available
          categories by default or categories available to the p_userToShowCategoriesFor userId.
          Ony a Manager/MeetingManager can give a userToShowCategoriesFor, by default it will be the currently
          connected user.
        '''
        portal = self.context
        member = api.user.get_current()
        tool = api.portal.get_tool('portal_plonemeeting')

        # passing a userToShowCategoriesFor is only available to 'Manager' and 'MeetingManager'
        if userToShowCategoriesFor:
            if not self._mayAccessAdvancedFunctionnalities():
                raiseMsg("You need to be 'Manager' or 'MeetingManager' to get available categories for a user!")
            # check that the passed userId exists...
            user = portal.acl_users.getUserById(userToShowCategoriesFor)
            if not user:
                raiseMsg("Trying to get avaialble categories for an unexisting user '%s'!" % userToShowCategoriesFor)

        # MeetingConfigs
        config_infos = []
        for config in tool.getActiveConfigs():
            configInfo = ConfigInfo()
            configInfo._UID = config.UID()
            configInfo._id = config.getId()
            configInfo._title = config.Title()
            configInfo._description = config.Description()
            configInfo._itemPositiveDecidedStates = config.getItemPositiveDecidedStates()
            # only return categories if the meetingConfig uses it
            if showCategories and not config.getUseGroupsAsCategories():
                for category in config.getCategories(userId=userToShowCategoriesFor):
                    basicInfo = BasicInfo()
                    basicInfo._UID = category.UID()
                    basicInfo._id = category.getId()
                    basicInfo._title = category.Title()
                    basicInfo._description = category.Description()
                    configInfo._categories.append(basicInfo)
            config_infos.append(configInfo)

        # organizations
        group_infos = []
        for org in get_organizations():
            groupInfo = GroupInfo()
            groupInfo._UID = org.UID()
            groupInfo._id = org.getId()
            groupInfo._title = org.Title()
            groupInfo._description = org.Description()
            group_infos.append(groupInfo)

        memberId = member.getId()
        logger.info('Configuration parameters at "%s" SOAP accessed by "%s".' %
                    (tool.absolute_url_path(), memberId))
        return config_infos, group_infos

    def _getUserInfos(self, userId, showGroups, suffix=None):
        '''
          Returns informations about the given userId.  If p_showGroups is True,
          it will also returns the list of organizations the user is member of.
        '''
        member = api.user.get_current()
        memberId = member.getId()

        # a member can get infos for himself
        # if we want to query informations for another user, the connected user
        # must have the 'MeetingManager' or 'Manager' role
        if not memberId == userId:
            if not self._mayAccessAdvancedFunctionnalities():
                raiseMsg("You need to be 'Manager' or 'MeetingManager' to get "
                         "user informations for another user than '%s'!" % memberId)

        # if getting user informations about the currently connected user
        # or the connected user is MeetingManager/Manager, proceed!
        user = api.user.get(userId)

        if not user:
            raiseMsg("Trying to get user informations for an unexisting user '%s'!" % userId)

        # show groups the user is member of if specified
        userGroups = []
        if showGroups:
            # if a particular suffix is defined, we use it, it will
            # returns only groups the user is member of with the defined
            # suffix, either it will returns every groups the user is member of
            tool = api.portal.get_tool('portal_plonemeeting')
            # backward compatibility,   _user received 'suffix' before but 'suffixes' now
            suffixes = suffix and [suffix] or []
            orgs = tool.get_orgs_for_user(user_id=userId, suffixes=suffixes, the_objects=True)
            for org in orgs:
                basicInfo = BasicInfo()
                basicInfo._UID = org.UID()
                basicInfo._id = org.getId()
                basicInfo._title = org.Title()
                basicInfo._description = org.Description()
                userGroups.append(basicInfo)

        return user.getProperty('fullname'), user.getProperty('email'), userGroups

    def _getItemInfos(self,
                      searchParams,
                      showExtraInfos=False,
                      showAnnexes=False,
                      allowed_annexes_types=[],
                      include_annex_binary=True,
                      showAssembly=False,
                      showTemplates=False,
                      inTheNameOf=None):
        '''
          Get an item with given searchParams dict.
          As the user is connected, the security in portal_catalog do the job.
        '''
        portal = self.context

        member = api.user.get_current()

        params = {}
        # remove leading '_' in searchParams
        for elt in searchParams.keys():
            searchParam = searchParams[elt]
            if searchParam:
                params[elt[1:]] = searchParam

        # check if we received at least one search parameter because calling the portal_catalog without search parameter
        # will return the entire catalog (even if we subforce using 'MeetingItem' meta_type here above)
        if not params:
            raiseMsg("Define at least one search parameter!")

        # if we specify in the request that we want to get infos about an item
        # for another user, we need to check that :
        # - user getting infos for another is 'MeetingManager' or 'Manager'
        # - the user we want to get informations for exists
        if inTheNameOf:
            if not self._mayAccessAdvancedFunctionnalities():
                raiseMsg("You need to be 'Manager' or 'MeetingManager' to get item informations 'inTheNameOf'!")
            # change considered member to inTheNameOf given userid
            member = portal.acl_users.getUserById(inTheNameOf)
            if not member:
                raiseMsg("Trying to get item informations 'inTheNameOf' an unexisting user '%s'!" % inTheNameOf)
        memberId = member.getId()

        tool = api.portal.get_tool('portal_plonemeeting')
        mc = None
        if 'meetingConfigId' in params and 'portal_type' not in params:
            # check that the given meetingConfigId exists
            meetingConfigId = params['meetingConfigId']
            mc = getattr(tool, meetingConfigId, None)
            if not mc or not mc.meta_type == 'MeetingConfig':
                raiseMsg("Unknown meetingConfigId : '%s'!" % meetingConfigId)
            params['portal_type'] = mc.getItemTypeName()

        # if we are getting item informations inTheNameOf, use this user for the rest of the process
        res = []
        try:
            if inTheNameOf:
                oldsm, original_auth_user = setup_user_in_the_name_of(portal, member)

            # force to use the 'MeetingItem' meta_type to be sure that attributes here above exist on found elements
            params['meta_type'] = 'MeetingItem'
            catalog = api.portal.get_tool('portal_catalog')
            uid_catalog = api.portal.get_tool('uid_catalog')
            params['sort_on'] = 'created'
            brains = catalog(**params)
            noDate = DateTime('1950/01/01 00:00:00 UTC')
            wfTool = api.portal.get_tool('portal_workflow')
            for brain in brains:
                # XXX for now we still need to wake up the item because we do not have the meeting's date
                # on the brain, this could be great to manage ticket http://trac.imio.be/trac/ticket/4176 so
                # we could avoid waking up the item if showExtraInfos is False
                item = brain.getObject()
                itemInfo = ItemInfo()
                itemInfo._UID = item.UID()
                itemInfo._id = item.getId()
                itemInfo._title = item.Title()
                itemInfo._creator = item.Creator()
                itemInfo._creation_date = localtime(item.created())
                itemInfo._modification_date = localtime(item.modified())
                # keep the original category/proposingGroup magic
                category_or_proposing_group = item.getCategory() and \
                    item.getCategory(theObject=True) or item.getProposingGroup(theObject=True)
                itemInfo._category = category_or_proposing_group.getId()
                itemInfo._description = item.getRawDescription()
                itemInfo._detailedDescription = item.getRawDetailedDescription()
                itemInfo._decision = item.getRawDecision(keepWithNext=False)
                preferred = item.getPreferredMeeting()
                itemInfo._preferredMeeting = not preferred == ITEM_NO_PREFERRED_MEETING_VALUE and preferred or ''
                preferredMeeting_brains = uid_catalog.searchResults(UID=preferred)
                preferredMeeting = preferredMeeting_brains and preferredMeeting_brains[0].getObject() or None
                itemInfo._preferred_meeting_date = \
                    preferredMeeting and preferredMeeting.date.utctimetuple() or localtime(noDate)
                itemInfo._review_state = wfTool.getInfoFor(item, 'review_state')
                meeting = item.hasMeeting() and item.getMeeting()
                itemInfo._meeting = meeting and meeting.UID() or None
                itemInfo._meeting_date = meeting and meeting.date.utctimetuple() or localtime(noDate)
                itemInfo._absolute_url = item.absolute_url()
                itemInfo._externalIdentifier = item.getField('externalIdentifier').getAccessor(item)()
                itemInfo._extraInfos = {}
                if showExtraInfos:
                    extraInfosFields = self._getExtraInfosFields(item)
                    # store every other informations in the 'extraInfos' dict
                    for field in extraInfosFields:
                        itemInfo._extraInfos[field.getName()] = field.getRaw(item)
                    # also add informations about the linked MeetingConfig
                    if not mc:
                        mc = tool.getMeetingConfig(item)
                    itemInfo._extraInfos['meeting_config_id'] = mc.getId()
                    itemInfo._extraInfos['meeting_config_title'] = mc.Title()
                    # add the review_state translated
                    itemInfo._extraInfos['review_state_translated'] = translate(msgid=itemInfo._review_state,
                                                                                domain='plone',
                                                                                context=portal.REQUEST)
                    # add the category title
                    itemInfo._extraInfos['category_title'] = category_or_proposing_group.Title()
                    # add the creator fullname
                    itemInfo._extraInfos['creator_fullname'] = tool.getUserName(itemInfo._creator)
                if showAnnexes:
                    for annex in get_annexes(item):
                        # filter on annexes types
                        if allowed_annexes_types and annex.content_category not in allowed_annexes_types:
                            continue
                        annexInfo = AnnexInfo()
                        annexInfo._id = annex.id
                        annexInfo._title = annex.Title()
                        annexInfo._annexTypeId = annex.content_category
                        annexInfo._filename = annex.file.filename
                        if include_annex_binary:
                            annexInfo._file = annex.file.data
                        itemInfo._annexes.append(annexInfo)
                if showAssembly and meeting:
                    # assembly or attendees?  Return a printed representation
                    assembly_lines = []
                    if item.getItemAssembly():
                        for field in item.Schema().filterFields(isMetadata=False):
                            field_name = field.getName()
                            if field_name.startswith('itemAssembly') and \
                               field_name != 'itemAssemblyGuests':
                                assembly_lines.append(field_name)
                                assembly_lines.append(field.getAccessor(item)())
                    else:
                        # contacts
                        # attendees
                        assembly_lines.append('Attendees')
                        attendees = item.get_attendees(the_objects=True)
                        attendees_lines = []
                        for attendee in attendees:
                            attendees_lines.append(attendee.get_short_title())
                        assembly_lines.append('\n'.join(attendees_lines))

                        # absents
                        assembly_lines.append('Absents')
                        absents = item.get_item_absents(the_objects=True)
                        absents_lines = []
                        for absent in absents:
                            absents_lines.append(absent.get_short_title())
                        assembly_lines.append('\n'.join(absents_lines))

                        # excused
                        assembly_lines.append('Excused')
                        excused = item.get_item_excused(the_objects=True)
                        excused_lines = []
                        for excused_contact in excused:
                            excused_lines.append(excused_contact.get_short_title())
                        assembly_lines.append('\n'.join(excused_lines))
                    # itemAssemblyGuests used for both
                    field = item.Schema()['itemAssemblyGuests']
                    assembly_lines.append('itemAssemblyGuests')
                    assembly_lines.append(field.getAccessor(item)())
                    itemInfo._item_assembly = '|'.join(assembly_lines)
                if showTemplates:
                    if not mc:
                        # we need the item's meetingConfig
                        mc = tool.getMeetingConfig(item)
                    templates = self._availablePodTemplates(item)
                    for template in templates:
                        for pod_format in template.pod_formats:
                            templateInfo = TemplateInfo()
                            templateInfo._title = template.Title()
                            templateInfo._templateFormat = pod_format
                            templateInfo._templateId = POD_TEMPLATE_ID_PATTERN.format(
                                template.getId(), pod_format)
                            # in case we are reusing another pod_file,
                            # use the get_file method that manages that
                            templateInfo._templateFilename = template.get_file().filename
                            itemInfo._templates.append(templateInfo)
                logger.info('Item at %s SOAP accessed by "%s".' %
                            (item.absolute_url_path(), memberId))
                res.append(itemInfo,)
        finally:
            # fallback to original user calling the SOAP method
            if inTheNameOf:
                teardown_user_in_the_name_of(portal, oldsm, original_auth_user)
        return res

    def _getItemTemplate(self, itemUID, templateId, inTheNameOf):
        '''
          Generates a POD template p_templateId on p_itemUID
        '''
        portal = self.context
        member = api.user.get_current()
        # if we specify in the request that we want to get a template of an item
        # for another user, we need to check that :
        # - user getting the template is 'MeetingManager' or 'Manager'
        # - the user we want to get informations for exists
        if inTheNameOf:
            if not self._mayAccessAdvancedFunctionnalities():
                raiseMsg("You need to be 'Manager' or 'MeetingManager' to get a template for an item 'inTheNameOf'!")
            # change considered member to inTheNameOf given userid
            member = portal.acl_users.getUserById(inTheNameOf)
            if not member:
                raiseMsg("Trying to create an item 'inTheNameOf' an unexisting user '%s'!" % inTheNameOf)

        # if we are creating an item inTheNameOf, use this user for the rest of the process
        try:
            if inTheNameOf:
                oldsm, original_auth_user = setup_user_in_the_name_of(portal, member)

            # search for the item, this will also check if the user can actually access it
            catalog = api.portal.get_tool('portal_catalog')
            brains = catalog(UID=itemUID)
            if not brains:
                raiseMsg("You can not access this item!")

            # check that the template is available to the member
            item = brains[0].getObject()
            templates = self._availablePodTemplates(item)
            theTemplate = None
            for template in templates:
                if template.getId() == templateId.split(POD_TEMPLATE_ID_PATTERN.format('', ''))[0]:
                    theTemplate = template
                    break
            if not theTemplate:
                raiseMsg("You can not access this template!")

            # we can access the item and the template, proceed!
            # generate the template and return the result
            logger.info('Template at "%s" for item at "%s" SOAP accessed by "%s".' %
                        (template.absolute_url_path(), item.absolute_url_path(), member.getId()))
            try:
                view = item.restrictedTraverse('@@document-generation')
                item.REQUEST.set('template_uid', theTemplate.UID())
                item.REQUEST.set('output_format', templateId.split(POD_TEMPLATE_ID_PATTERN.format('', ''))[1])
                res = view()
            except Exception, e:
                raiseMsg("Exception : %s" % e.message)
        finally:
            # fallback to original user calling the SOAP method
            if inTheNameOf:
                teardown_user_in_the_name_of(portal, oldsm, original_auth_user)
        return res

    def _getExtraInfosFields(self, item):
        """
          Returns fields considered as 'extraInfos' aka not main informations
        """
        res = []
        for field in item.schema.filterFields(isMetadata=False):
            if field.getName() not in MAIN_DATA_FROM_ITEM_SCHEMA:
                res.append(field)
        return res

    def _meetingsAcceptingItems(self, meetingConfigId, inTheNameOf=None):
        '''
        '''
        portal = self.context
        member = api.user.get_current()
        tool = api.portal.get_tool('portal_plonemeeting')

        # if we specify in the request that we want to get infos about an item
        # for another user, we need to check that :
        # - user getting infos for another is 'MeetingManager' or 'Manager'
        # - the user we want to get informations for exists
        if inTheNameOf:
            if not self._mayAccessAdvancedFunctionnalities():
                raiseMsg("You need to be 'Manager' or 'MeetingManager' to get item informations 'inTheNameOf'!")
            # change considered member to inTheNameOf given userid
            member = portal.acl_users.getUserById(inTheNameOf)
            if not member:
                raiseMsg("Trying to get meetings accepting items 'inTheNameOf' an unexisting user '%s'!" % inTheNameOf)
        memberId = member.getId()

        cfg = getattr(tool, meetingConfigId or '', None)
        if not cfg or not cfg.meta_type == 'MeetingConfig':
            raiseMsg("Unknown meetingConfigId : '%s'!" % meetingConfigId)

        # if we are getting item informations inTheNameOf, use this user for the rest of the process
        res = []
        try:
            if inTheNameOf:
                oldsm, original_auth_user = setup_user_in_the_name_of(portal, member)

            brains = cfg.getMeetingsAcceptingItems()
            for brain in brains:
                # XXX for now we still need to wake up the item because we do not have the meeting's date
                # on the brain, this could be great to manage ticket http://trac.imio.be/trac/ticket/4176 so
                # we could avoid waking up the item if showExtraInfos is False
                meeting = brain.getObject()
                meetingInfo = MeetingInfo()
                meetingInfo._UID = meeting.UID()
                meetingInfo._date = meeting.date.utctimetuple()
                res.append(meetingInfo,)
        finally:
            # fallback to original user calling the SOAP method
            if inTheNameOf:
                teardown_user_in_the_name_of(portal, oldsm, original_auth_user)

        logger.info('MeetingConfig at %s SOAP accessed by "%s" to get meetings accepting items.' %
                    (cfg.absolute_url_path(), memberId))
        return res

    def _createItem(self,
                    meetingConfigId,
                    proposingGroupId,
                    creationData,
                    cleanHtml=True,
                    wfTransitions=[],
                    inTheNameOf=None):
        '''
          Create an item with given parameters
        '''
        portal = self.context
        tool = api.portal.get_tool('portal_plonemeeting')

        warnings = []
        member = api.user.get_current()

        # if we specify in the request that we want to create an item
        # for another user, we need to check that :
        # - user creating for another is 'MeetingManager' or 'Manager'
        # - the user we want to create an item for exists
        if inTheNameOf:
            if not self._mayAccessAdvancedFunctionnalities():
                raiseMsg("You need to be 'Manager' or 'MeetingManager' to create an item 'inTheNameOf'!")
            # change considered member to inTheNameOf given userid
            member = portal.acl_users.getUserById(inTheNameOf)
            if not member:
                raiseMsg("Trying to create an item 'inTheNameOf' an unexisting user '%s'!" % inTheNameOf)
        memberId = member.getId()

        # check that the given meetingConfigId exists
        cfg = getattr(tool, meetingConfigId, None)
        if not cfg or not cfg.meta_type == 'MeetingConfig':
            raiseMsg("Unknown meetingConfigId : '%s'!" % meetingConfigId)

        # check that the user is a creator for given proposingGroupId
        # proposingGroupId may be an organization id (backward compatibility) or an organization UID
        # get the MeetingGroups for wich inTheNameOfMemberId is creator
        proposingGroupUID = org_id_to_uid(proposingGroupId, raise_on_error=False)
        if not proposingGroupUID:
            proposingGroupUID = proposingGroupId
        userOrgUids = tool.get_orgs_for_user(
            user_id=memberId, suffixes=['creators'], the_objects=False)
        if proposingGroupUID not in userOrgUids:
            raiseMsg("'%s' can not create items for the '%s' group!" % (memberId, proposingGroupId))

        # title is mandatory!
        if not creationData.__dict__['_title']:
            raiseMsg("A 'title' is mandatory!")

        # build creationData
        # creationData keys begin with an '_' (_title, _description, ...) so tranform them
        data = {}
        for elt in creationData.__dict__.keys():
            # do not take annexes into account
            if not elt == '_annexes':
                data[elt[1:]] = creationData.__dict__[elt]

        # category can not be None
        if data['category'] is None:
            data['category'] = ''

        # ignore boolean 'toDiscuss' that is None, it means it was not set
        # or if value is set during item present
        if data['toDiscuss'] is None:
            data.pop('toDiscuss')

        # raise if we pass an optional attribute that is not activated in this MeetingConfig
        optionalItemFields = cfg.listUsedItemAttributes()
        activatedOptionalItemFields = cfg.getUsedItemAttributes()
        for field in data:
            # if the field is an optional field that is not used and that has a value (contains data), we raise
            if field in optionalItemFields and \
               field not in activatedOptionalItemFields and \
               data[field]:
                raiseMsg("The optional field \"%s\" is not activated in this configuration!" % field)

        # raise if we pass a preferredMeeting that is not a meeting accepting items
        if not data['preferredMeeting']:
            data['preferredMeeting'] = ITEM_NO_PREFERRED_MEETING_VALUE
        if not data['preferredMeeting'] == ITEM_NO_PREFERRED_MEETING_VALUE and \
           not data['preferredMeeting'] in \
           [meetingBrain.UID for meetingBrain in cfg.getMeetingsAcceptingItems()]:
            raiseMsg("The given preferred meeting UID ({0}) is not a meeting accepting items!".format(
                data['preferredMeeting']))

        # validate passed associatedGroups
        associatedGroups = data['associatedGroups']
        if associatedGroups:
            vocab = get_vocab(cfg, 'Products.PloneMeeting.vocabularies.associatedgroupsvocabulary')
            ag_term_keys = vocab.by_token.keys()
            difference = tuple(set(associatedGroups).difference(ag_term_keys))
            if difference:
                raiseMsg("The \"associatedGroups\" data contains wrong values: \"{0}\"!".format(
                    ', '.join(difference)))

        # validate passed groupsInCharge
        groupsInCharge = data['groupsInCharge']
        if groupsInCharge:
            vocab = get_vocab(cfg, 'Products.PloneMeeting.vocabularies.groupsinchargevocabulary')
            gic_term_keys = vocab.by_token.keys()
            difference = tuple(set(groupsInCharge).difference(gic_term_keys))
            if difference:
                raiseMsg("The \"groupsInCharge\" data contains wrong values: \"{0}\"!".format(
                    ', '.join(difference)))

        # validate passed optionalAdvisers
        optionalAdvisers = data['optionalAdvisers']
        if optionalAdvisers:
            if not cfg.getUseAdvices():
                raiseMsg("The advices functionnality is not enabled for this configuration!")

            # advices are enabled, check if given values are correct
            vocab = get_vocab(cfg, 'Products.PloneMeeting.vocabularies.itemoptionaladvicesvocabulary',
                              **{'include_selected': False, 'include_not_selectable_values': False})
            oa_term_keys = vocab.by_token.keys()
            difference = tuple(set(optionalAdvisers).difference(oa_term_keys))
            if difference:
                raiseMsg("The \"optionalAdvisers\" data contains wrong values: \"{0}\"!".format(
                    ', '.join(difference)))

        # externalIdentifier is indexed, it can not be None
        if not data['externalIdentifier']:
            data['externalIdentifier'] = ''

        # manage extraAttrs, check if it exist, if so, add it to data
        # for now, we only accept TextFields
        for extraAttr in creationData._extraAttrs:
            key = extraAttr._key
            value = extraAttr._value
            # the given extraAttr must be in the item schema
            if key not in MeetingItem.schema:
                raiseMsg("The extraAttr '%s' was not found the the MeetingItem schema!" % key)
            field = MeetingItem.schema[key]
            # only support XHTML TextField at the moment, aka field using RichWidget
            # others will need validation
            if not isinstance(field.widget, RichWidget):
                raiseMsg("The extraAttr '%s' must correspond to a field using a "
                         "'RichWidget' in the MeetingItem schema!" % key)
            # avoid overriding a value
            if key not in data:
                data[key] = value

        try:
            # if we are creating an item inTheNameOf, use this user for the rest of the process
            if inTheNameOf:
                oldsm, original_auth_user = setup_user_in_the_name_of(portal, member)

            # get or create the meetingFolder the item will be created in
            # if the user does not have a memberArea
            # (never connected, then we raise an error)
            destFolder = tool.getPloneMeetingFolder(meetingConfigId, memberId)
            if destFolder.meta_type == 'Plone Site':
                raiseMsg("No member area for '%s'.  Never connected to PloneMeeting?" % memberId)

            type_name = cfg.getItemTypeName()
            data.update({'proposingGroup': proposingGroupUID,
                         'id': portal.generateUniqueId(type_name), })

            # find htmlFieldIds we will have to check/clean
            # RichText fields are not handled by invokeFactory so we
            # clean it then set it...
            htmlFieldIds = []
            managedFieldIds = data.keys()
            for field in MeetingItem.schema.fields():
                fieldName = field.getName()
                if fieldName in managedFieldIds and field.widget.getName() in ['RichWidget', 'VisualWidget', ]:
                    htmlFieldIds.append(fieldName)
            warnWrongHTML = False
            if cleanHtml:
                cleaner = Cleaner()
                for htmlFieldId in htmlFieldIds:
                    # BeautifulSoup does not deal with NoneType
                    if data[htmlFieldId] is None:
                        data[htmlFieldId] = ''
                    soup = BeautifulSoup(safe_unicode(data[htmlFieldId]))
                    # we need a surrounding <p></p> or the content is not generated by appy.pod
                    if not data[htmlFieldId].startswith('<p>') or not data[htmlFieldId].endswith('</p>'):
                        data[htmlFieldId] = '<p>%s</p>' % data[htmlFieldId]
                    if not soup.contents or not getattr(soup.contents[0], 'name', None) == u'p':
                        soup = BeautifulSoup(safe_unicode(data[htmlFieldId]))
                    renderedSoupContents = soup.renderContents()
                    if not isinstance(renderedSoupContents, unicode):
                        renderedSoupContents = unicode(renderedSoupContents, 'utf-8')
                    # clean HTML with HTMLParser, it will remove special entities like &#xa0;
                    renderedSoupContents = HTMLParser().unescape(renderedSoupContents)
                    # clean HTML with lxml Cleaner
                    renderedSoupContents = cleaner.clean_html(renderedSoupContents).encode('utf-8')
                    # clean_html surrounds the cleaned HTML with <div>...</div>... removes it!
                    if renderedSoupContents.startswith('<div>') and renderedSoupContents.endswith('</div>'):
                        renderedSoupContents = renderedSoupContents[5:-6]
                    if not renderedSoupContents == data[htmlFieldId]:
                        warnWrongHTML = True
                        data[htmlFieldId] = renderedSoupContents

            # check that if category is mandatory (getUseGroupsAsCategories is False), it is given
            # and that given category is available
            # if we are not using categories, just ensure that we received an empty category
            availableCategories = not cfg.getUseGroupsAsCategories() and \
                [cat.getId() for cat in cfg.getCategories()] or ['', ]
            if not data['category'] in availableCategories:
                # special message if category mandatory and not given
                if not cfg.getUseGroupsAsCategories() and not data['category']:
                    raiseMsg("In this config, category is mandatory!")
                elif cfg.getUseGroupsAsCategories() and data['category']:
                    raiseMsg("This config does not use categories, the given '%s' "
                             "category can not be used!" % data['category'])
                # we are using categories but the given one is not in availableCategories
                elif not cfg.getUseGroupsAsCategories():
                    raiseMsg("'%s' is not available for the '%s' group!" % (
                        data['category'], proposingGroupId))

            # we create the item to be able to check the category here above...
            itemId = destFolder.invokeFactory(type_name, **data)
            item = getattr(destFolder, itemId)
            item.setCategory(data['category'])

            # add a record to the item workflow_history to specify that item was created thru SOAP WS
            action_name = ITEM_SOAP_CREATED
            action_label = action_name + '_comments'
            add_wf_history_action(item,
                                  action_name=action_name,
                                  action_label=action_label,
                                  user_id=memberId)

            # processForm calls at_post_create_script too
            # this is necessary before adding annexes
            item.at_post_create_script()

            # HTML fields were not set by invokeFactory, set it...
            for htmlFieldId in htmlFieldIds:
                # use 'text/x-html-safe' mimetype when creating the item
                field = item.getField(htmlFieldId)
                field.getMutator(item)(data[htmlFieldId], mimetype='text/x-html-safe')

            # manage externalIdentifier
            externalIdentifier = False
            field = item.getField(EXTERNAL_IDENTIFIER_FIELD_NAME)
            # can not be None as it is indexed
            externalIdentifier = data['externalIdentifier'] or ''
            if data['externalIdentifier']:
                # we received an externalIdentifier, use it!
                field.getMutator(item)(data['externalIdentifier'])
                externalIdentifier = True
            else:
                field.getMutator(item)(field.default)

            # manage annexes
            # call processForm before adding annexes because it calls at_post_create_script
            # where we set some usefull values regarding annexes
            item.processForm()
            # add warning message after processForm because the id of the item may be changed
            if warnWrongHTML:
                warning_message = translate(WRONG_HTML_WARNING,
                                            domain='imio.pm.ws',
                                            mapping={'item_path': item.absolute_url_path(),
                                                     'creator': memberId},
                                            context=portal.REQUEST)
                logger.warning(warning_message)
                warnings.append(warning_message)

            # existing annex types
            annexTypeIds = cfg.annexes_types.item_annexes.objectIds()
            for annex in creationData._annexes:
                annex_title = annex._title
                annex_type_id = annex._annexTypeId
                annex_filename = annex._filename
                validFileName = annex_filename and len(annex_filename.split('.')) == 2
                # if annex._file is None, we turn it to an empty string
                annex_file = annex._file or ''
                if not annex_type_id or annex_type_id not in annexTypeIds:
                    # take the first available annexType that is the default one
                    annex_type_id = annexTypeIds[0]
                annex_type = getattr(cfg.annexes_types.item_annexes, annex_type_id)
                # manage mimetype manually
                # as we receive base64 encoded binary, mimetypes registry can not handle this correctly...
                mr = self.context.mimetypes_registry
                mime = magic.Magic(mime=True)
                magic_mimetype = None
                try:
                    magic_mimetype = mime.from_buffer(annex_file)
                except MagicException:
                    # in case there is an error with magic trying to find annex mimetype, we pass
                    # we will have magic_mimetype=None and so will try to use file extension to
                    # determinate it here under
                    pass
                mr_mimetype = ()
                if magic_mimetype:
                    mr_mimetype = mr.lookup(magic_mimetype)
                else:
                    # if libmagic could not determine file mimetype (like in version 5.09 of the command 'file'
                    # where MS mimetypes (doc, xls, ...) are not recognized...), we use the file extension...
                    if validFileName:
                        # mr.lookup here above returns a tuple so we build a tuple also...
                        mr_mimetype = (mr.lookupExtension(annex_filename.split('.')[1]),)
                # check if a mimetype has been found and if a file extension was defined for it
                if not mr_mimetype:
                    warning_message = translate(MIMETYPE_NOT_FOUND_OF_ANNEX_WARNING,
                                                domain='imio.pm.ws',
                                                mapping={'annex_path': safe_unicode(annex_filename or annex_title),
                                                         'item_path': item.absolute_url_path()},
                                                context=portal.REQUEST)
                    logger.warning(warning_message)
                    warnings.append(warning_message)
                    continue
                elif not mr_mimetype[0].extensions:
                    warning_message = translate(NO_EXTENSION_FOR_MIMETYPE_OF_ANNEX_WARNING,
                                                domain='imio.pm.ws',
                                                mapping={'mime': mr_mimetype[0].normalized(),
                                                         'annex_path': unicode((annex_filename or annex_title),
                                                                               'utf-8'),
                                                         'item_path': item.absolute_url_path()},
                                                context=portal.REQUEST)
                    logger.warning(warning_message)
                    warnings.append(warning_message)
                    continue
                elif len(mr_mimetype[0].extensions) > 1:
                    if not validFileName:
                        # several extensions are proposed by mimetypes_registry
                        # and we have nothing to find out what is the extension to use
                        warning_message = translate(MULTIPLE_EXTENSION_FOR_MIMETYPE_OF_ANNEX_WARNING,
                                                    domain='imio.pm.ws',
                                                    mapping={'mime': mr_mimetype[0].normalized(),
                                                             'annex_path': unicode((annex_filename or annex_title),
                                                                                   'utf-8'),
                                                             'item_path': item.absolute_url_path()},
                                                    context=portal.REQUEST)
                        logger.warning(warning_message)
                        warnings.append(warning_message)
                        continue
                # now that we have the correct mimetype, we can handle the filename if necessary
                if not validFileName:
                    # we have the file extension, generate a filename
                    annex_filename = "annex.%s" % mr_mimetype[0].extensions[0]
                # now that we have everything we need, proceed with annex creation
                createContentInContainer(
                    container=item,
                    portal_type='annex',
                    title=annex_title,
                    file=namedfile.NamedBlobFile(annex_file,
                                                 filename=safe_unicode(annex_filename)),
                    content_category=calculate_category_id(annex_type),
                    to_print=False,
                    confidential=False)

            # manage wfTransitions
            if wfTransitions:
                # trigger transitions
                wfTool = api.portal.get_tool('portal_workflow')
                wf_comment = 'wf_transition_triggered_by_application'
                with api.env.adopt_roles(roles=['Manager']):
                    for tr in wfTransitions:
                        available_transitions = [t['id'] for t in wfTool.getTransitionsFor(item)]
                        if tr not in available_transitions:
                            warning_message = "While treating wfTransitions, could not " \
                                "trigger the '{0}' transition!".format(tr)
                            # additional info if failed to trigger the 'present' transition
                            if tr == 'present':
                                warning_message += " Make sure a meeting accepting items " \
                                    "exists in configuration '{0}'!".format(cfg.getId())
                            warnings.append(warning_message)
                            continue
                        # we are sure transition is available, trigger it
                        wfTool.doActionFor(item, tr, comment=wf_comment)

            # log finally
            logger.info('Item at "%s"%s SOAP created by "%s".' %
                        (item.absolute_url_path(),
                         (externalIdentifier and ' with externalIdentifier "%s"' %
                          item.externalIdentifier or ''), memberId))
        finally:
            # fallback to original user calling the SOAP method
            if inTheNameOf:
                teardown_user_in_the_name_of(portal, oldsm, original_auth_user)
        return item.UID(), warnings

    def _mayAccessAdvancedFunctionnalities(self):
        '''
          This method will protect various advances functionnalities like the
          'inTheNameOf' functionnality.
          By default, the given user must be 'MeetingManager' for a
          MeetingConfig to be able to use 'inTheNameOf' or a 'Manager'.
        '''
        tool = api.portal.get_tool('portal_plonemeeting')
        if tool.userIsAmong(['meetingmanagers']) or tool.isManager(realManagers=True):
            return True

        return False

    def _availablePodTemplates(self, item):
        """ """
        viewlet = PMDocumentGeneratorLinksViewlet(item,
                                                  item.REQUEST,
                                                  None,
                                                  None)
        return viewlet.get_generable_templates()


class WS4PMWSDL(BrowserView):
    """
      This render the SOAP/WSDL depending on the current portal_url
    """

    def __call__(self, dump_wsdl=False):
        """ """
        if dump_wsdl:
            self._dump_wsdl()
        return self.index()

    def _dump_wsdl(self):
        """To generate ZSI client/server/types files, we need the WSDL
           to be stored on the filesystem.
           This will dump the WSDL under name dumpedWSDL.txt."""
        portal = api.portal.get()
        rendered = self.index()
        rendered = rendered.replace(portal.absolute_url(), 'http://ws4pm.imio.be')
        rendered = rendered.replace(
            '<?xml version="1.0" encoding="utf-8"?>\n',
            '<?xml version="1.0" encoding="utf-8"?>\n'
            '<!-- This file is generated by calling http://portal_url/@@ws4pm.wsdl?dump_wsdl:boolean=True -->\n')
        dumpedWSDL = open(os.path.dirname(__file__) + '/../dumpedWSDL.txt', 'wb')
        dumpedWSDL.write("" + rendered)
        dumpedWSDL.close()



# -*- coding: utf-8 -*-

from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase


class testSetup(WS4PMTestCase):
    """
        Tests the setup : install/uninstall process and WSDL
    """

    def test_ws_uninstall(self):
        """
          At uninstall time, we remove the BrowserLayer that makes
          schemaextender and ws4pm.wsdl available.  We also remove the added indexes/columns.
        """
        # by default, the WSDL is available and the schemaextender
        self.assertEquals(self.portal.restrictedTraverse('@@ws4pm.wsdl').context.absolute_url(),
                          self.portal.absolute_url())
        self.changeUser('pmManager')
        item = self.create('MeetingItem')
        self.failUnless('externalIdentifier' in item.Schema().keys())
        # the BrowserLayer is registered
        from imio.pm.ws.interfaces import IWS4PMLayer
        from plone.browserlayer.utils import registered_layers
        self.failUnless(IWS4PMLayer in registered_layers())
        # portal_catalog contains an index and a metadata for "externalIdentifier"
        self.failUnless("externalIdentifier" in self.portal.portal_catalog.indexes())
        self.failUnless("externalIdentifier" in self.portal.portal_catalog.schema())
        # now uninstall imio.pm.ws
        self.portal.portal_setup.runAllImportStepsFromProfile('profile-imio.pm.ws:uninstall')
        # no more schemaextender
        self.failIf('externalIdentifier' in item.Schema().keys())
        # the WSDL is no more available
        # this does not seem to work in tests?  While uninstalling the BrowserLayer
        # the BrowserView ws4pm.wsdl is still available... ???
        # but the BrowserLayer is no more registered...
        self.failIf(IWS4PMLayer in registered_layers())
        # indexes/metadatas are removed from portal_catalog
        self.failIf("externalIdentifier" in self.portal.portal_catalog.indexes())
        self.failIf("externalIdentifier" in self.portal.portal_catalog.schema())


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSetup, prefix='test_ws_'))
    return suite



from setuptools import setup, find_packages
import os

version = '3.6'

setup(name='imio.pm.ws',
      version=version,
      description="WebServices for PloneMeeting",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("CHANGES.rst")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
          "Development Status :: 6 - Mature",
          "Framework :: Plone",
          "Framework :: Plone :: 4.3",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
        ],
      keywords='',
      author='Gauthier Bastien',
      author_email='gauthier@imio.be',
      url='http://svn.communesplone.org/svn/communesplone/imio.pm.ws/',
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['imio', 'imio.pm'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'lxml',
          'z3c.soap',
          'BeautifulSoup',
          'python-magic',
          'ZSI',
          'archetypes.schemaextender',
          'Products.PloneMeeting'
          # -*- optional SOAP clients for tests (see docs/README.txt) -*-
          #'SOAPpy',
          #'suds',
      ],
      dependency_links = ['http://sourceforge.net/projects/pywebsvcs/files/ZSI/ZSI-2.0/ZSI-2.0.tar.gz/download'],
      extras_require={'test': ['Products.PloneMeeting [test]']},
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



EXTERNAL_IDENTIFIER_FIELD_NAME = 'externalIdentifier'

MAIN_DATA_FROM_ITEM_SCHEMA = ['title',
                              'id',
                              'category',
                              'description',
                              'detailedDescription',
                              'decision',
                              'externalIdentifier', ]

POD_TEMPLATE_ID_PATTERN = '{0}__format__{1}'



# -*- coding: utf-8 -*-
#
# File: testcheckislinked.py
#
# Copyright (c) 2013 by Imio.be
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

from imio.helpers.cache import cleanRamCacheFor
from imio.pm.ws.soap.soapview import SOAPView
from imio.pm.ws.tests.WS4PMTestCase import WS4PMTestCase
from imio.pm.ws.WS4PM_client import checkIsLinkedRequest
from imio.pm.ws.WS4PM_client import checkIsLinkedResponse

import ZSI


class testSOAPCheckIsLinked(WS4PMTestCase):
    """
        Tests the soap.checkItemIsLinkedRequest method by accessing the real SOAP service
    """

    def test_ws_checkIsLinkedRequest(self):
        """
          Test that we can ckech that an item is linked to a given externalIdentifier even
          if the MeetingManager doing the can can not actually see the item... (we use unrestricted search)
        """
        # try as a non MeetingManager
        self.changeUser('pmCreator1')
        checkIsLinkedReq = checkIsLinkedRequest()
        checkIsLinkedResponseHolder = checkIsLinkedResponse()
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, checkIsLinkedReq).checkIsLinkedRequest(checkIsLinkedReq,
                                                                         checkIsLinkedResponseHolder)
        self.assertEquals(cm.exception.string,
                          "You need to be 'Manager' or 'MeetingManager' to check if an element is linked to an item!")
        # now create an item as 'pmCreator2', aka a user in a group 'pmManager' can not access
        self.changeUser('pmCreator2')
        req = self._prepareCreationData()
        req._proposingGroupId = 'vendors'
        req._creationData._externalIdentifier = 'my-external-identifier'
        newItem, response = self._createItem(req)
        # check that using getItemInfos, MeetingManager can not get informations about created item
        self.changeUser('pmManager')
        self.failIf(self._getItemInfos(newItem.UID(), toBeDeserialized=False)._itemInfo)
        # but while checking if an item is linked, it works...
        # first check for anotherexternalIdentifier
        checkIsLinkedReq._meetingConfigId = None
        checkIsLinkedReq._externalIdentifier = 'my-unexisting-external-identifier'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        res = SOAPView(self.portal, checkIsLinkedReq).checkIsLinkedRequest(checkIsLinkedReq,
                                                                           checkIsLinkedResponseHolder)
        self.assertFalse(res._isLinked)
        # passing a wrong meetingConfigId will raise a ZSI.Fault
        # first check for anotherexternalIdentifier
        checkIsLinkedReq._meetingConfigId = 'wrong-meeting-config-id'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        with self.assertRaises(ZSI.Fault) as cm:
            SOAPView(self.portal, checkIsLinkedReq).checkIsLinkedRequest(checkIsLinkedReq,
                                                                         checkIsLinkedResponseHolder)
        self.assertEquals(cm.exception.string,
                          "Unknown meetingConfigId : 'wrong-meeting-config-id'!")
        # now with the values corresponding to the created item
        checkIsLinkedReq._meetingConfigId = 'plonegov-assembly'
        checkIsLinkedReq._externalIdentifier = 'my-external-identifier'
        cleanRamCacheFor('Products.PloneMeeting.ToolPloneMeeting.userIsAmong')
        res = SOAPView(self.portal, checkIsLinkedReq).checkIsLinkedRequest(checkIsLinkedReq,
                                                                           checkIsLinkedResponseHolder)
        self.assertTrue(res._isLinked)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    # add a prefix because we heritate from testMeeting and we do not want every tests of testMeeting to be run here...
    suite.addTest(makeSuite(testSOAPCheckIsLinked, prefix='test_ws_'))
    return suite



# -*- coding: utf-8 -*-
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.testing import z2
from plone.testing import zca
from Products.PloneMeeting.testing import PMLayer

import imio.pm.ws


class WSLayer(PMLayer):
    """ """


WS4PM_ZCML = zca.ZCMLSandbox(filename="testing.zcml",
                             package=imio.pm.ws,
                             name='WS4PM_ZCML')

WS4PM_Z2 = z2.IntegrationTesting(bases=(z2.STARTUP, WS4PM_ZCML),
                                 name='WS4PM_Z2')

WS4PM_PM_TEST_PROFILE = WSLayer(
    zcml_filename="testing.zcml",
    zcml_package=imio.pm.ws,
    additional_z2_products=('imio.dashboard',
                            'Products.PloneMeeting',
                            'Products.CMFPlacefulWorkflow',
                            'Products.PasswordStrength'),
    gs_profile_id='Products.PloneMeeting:testing',
    name="WS4PM_PM_TEST_PROFILE")

WS4PM_PM_TEST_PROFILE_INTEGRATION = IntegrationTesting(
    bases=(WS4PM_PM_TEST_PROFILE,), name="WS4PM_PM_TEST_PROFILE_INTEGRATION")

WS4PM_PM_TEST_PROFILE_FUNCTIONAL = FunctionalTesting(
    bases=(WS4PM_PM_TEST_PROFILE,), name="WS4PM_PM_TEST_PROFILE_FUNCTIONAL")
